@@ -3,6 +3,7 @@ package genswagger
 import (
 	"bytes"
 	"encoding/json"
+	"fmt"
 
 	"github.com/grpc-ecosystem/grpc-gateway/protoc-gen-grpc-gateway/descriptor"
 )
@@ -25,6 +26,8 @@ type swaggerInfoObject struct {
 
 	Contact *swaggerContactObject `json:"contact,omitempty"`
 	License *swaggerLicenseObject `json:"license,omitempty"`
+
+	extensions []extension
 }
 
 // http://swagger.io/specification/#contactObject
@@ -46,13 +49,18 @@ type swaggerExternalDocumentationObject struct {
 	URL         string `json:"url,omitempty"`
 }
 
+type extension struct {
+	key   string
+	value json.RawMessage
+}
+
 // http://swagger.io/specification/#swaggerObject
 type swaggerObject struct {
 	Swagger             string                              `json:"swagger"`
 	Info                swaggerInfoObject                   `json:"info"`
 	Host                string                              `json:"host,omitempty"`
 	BasePath            string                              `json:"basePath,omitempty"`
-	Schemes             []string                            `json:"schemes"`
+	Schemes             []string                            `json:"schemes,omitempty"`
 	Consumes            []string                            `json:"consumes"`
 	Produces            []string                            `json:"produces"`
 	Paths               swaggerPathsObject                  `json:"paths"`
@@ -60,6 +68,8 @@ type swaggerObject struct {
 	SecurityDefinitions swaggerSecurityDefinitionsObject    `json:"securityDefinitions,omitempty"`
 	Security            []swaggerSecurityRequirementObject  `json:"security,omitempty"`
 	ExternalDocs        *swaggerExternalDocumentationObject `json:"externalDocs,omitempty"`
+
+	extensions []extension
 }
 
 // http://swagger.io/specification/#securityDefinitionsObject
@@ -75,6 +85,8 @@ type swaggerSecuritySchemeObject struct {
 	AuthorizationURL string              `json:"authorizationUrl,omitempty"`
 	TokenURL         string              `json:"tokenUrl,omitempty"`
 	Scopes           swaggerScopesObject `json:"scopes,omitempty"`
+
+	extensions []extension
 }
 
 // http://swagger.io/specification/#scopesObject
@@ -104,9 +116,12 @@ type swaggerOperationObject struct {
 	Parameters  swaggerParametersObject `json:"parameters,omitempty"`
 	Tags        []string                `json:"tags,omitempty"`
 	Deprecated  bool                    `json:"deprecated,omitempty"`
+	Produces    []string                `json:"produces,omitempty"`
 
 	Security     *[]swaggerSecurityRequirementObject `json:"security,omitempty"`
 	ExternalDocs *swaggerExternalDocumentationObject `json:"externalDocs,omitempty"`
+
+	extensions []extension
 }
 
 type swaggerParametersObject []swaggerParameterObject
@@ -133,9 +148,10 @@ type swaggerParameterObject struct {
 // core part of schema, which is common to itemsObject and schemaObject.
 // http://swagger.io/specification/#itemsObject
 type schemaCore struct {
-	Type   string `json:"type,omitempty"`
-	Format string `json:"format,omitempty"`
-	Ref    string `json:"$ref,omitempty"`
+	Type    string          `json:"type,omitempty"`
+	Format  string          `json:"format,omitempty"`
+	Ref     string          `json:"$ref,omitempty"`
+	Example json.RawMessage `json:"example,omitempty"`
 
 	Items *swaggerItemsObject `json:"items,omitempty"`
 
@@ -146,22 +162,39 @@ type schemaCore struct {
 	Default string   `json:"default,omitempty"`
 }
 
-type swaggerItemsObject schemaCore
-
-func (o *swaggerItemsObject) getType() string {
-	if o == nil {
-		return ""
+func (s *schemaCore) setRefFromFQN(ref string, reg *descriptor.Registry) error {
+	name, ok := fullyQualifiedNameToSwaggerName(ref, reg)
+	if !ok {
+		return fmt.Errorf("setRefFromFQN: can't resolve swagger name from '%v'", ref)
 	}
-	return o.Type
+	s.Ref = fmt.Sprintf("#/definitions/%s", name)
+	return nil
 }
 
+type swaggerItemsObject schemaCore
+
 // http://swagger.io/specification/#responsesObject
 type swaggerResponsesObject map[string]swaggerResponseObject
 
 // http://swagger.io/specification/#responseObject
 type swaggerResponseObject struct {
-	Description string              `json:"description"`
-	Schema      swaggerSchemaObject `json:"schema"`
+	Description string                 `json:"description"`
+	Schema      swaggerSchemaObject    `json:"schema"`
+	Examples    map[string]interface{} `json:"examples,omitempty"`
+	Headers     swaggerHeadersObject   `json:"headers,omitempty"`
+
+	extensions []extension
+}
+
+type swaggerHeadersObject map[string]swaggerHeaderObject
+
+// http://swagger.io/specification/#headerObject
+type swaggerHeaderObject struct {
+	Description string          `json:"description,omitempty"`
+	Type        string          `json:"type,omitempty"`
+	Format      string          `json:"format,omitempty"`
+	Default     json.RawMessage `json:"default,omitempty"`
+	Pattern     string          `json:"pattern,omitempty"`
 }
 
 type keyVal struct {
@@ -207,6 +240,7 @@ type swaggerSchemaObject struct {
 
 	ExternalDocs *swaggerExternalDocumentationObject `json:"externalDocs,omitempty"`
 
+	ReadOnly         bool     `json:"readOnly,omitempty"`
 	MultipleOf       float64  `json:"multipleOf,omitempty"`
 	Maximum          float64  `json:"maximum,omitempty"`
 	ExclusiveMaximum bool     `json:"exclusiveMaximum,omitempty"`
